searchData={"items":[{"type":"module","title":"Mozart.BpmProcess","doc":"This module implements a BPM Domain Specific Language (DSL) framework. To use, insert\n**use Mozart.BpmProcess** into your modules.\n\n```elixir\ndefmodule Mozart.MyBpmApplication do\n  use Mozart.BpmProcess\n\n  # Your process model definitions\nend\n```","ref":"Mozart.BpmProcess.html"},{"type":"macro","title":"Mozart.BpmProcess.case_i/2","doc":"Used to specify one of many alternate execution paths. The first argument is either\na captured function or an atom corresponding to local function of arity 1. The second\nargument is a block of tasks.\n\n```\ndefprocess \"two case process\" do\n  case_task \"yes or no\" do\n    case_i :x_less_than_y do\n      user_task(\"1\", group: \"admin\")\n      user_task(\"2\", group: \"admin\")\n    end\n    case_i :x_greater_or_equal_y do\n      user_task(\"3\", group: \"admin\")\n      user_task(\"4\", group: \"admin\")\n    end\n  end\nend\n```","ref":"Mozart.BpmProcess.html#case_i/2"},{"type":"macro","title":"Mozart.BpmProcess.case_task/2","doc":"Used to select one of many execution paths. Arguments are a task name and a list of cases.\n\n```\ndefprocess \"two case process\" do\n  case_task \"yes or no\" do\n    case_i :x_less_than_y do\n      user_task(\"1\", group: \"admin\")\n      user_task(\"2\", group: \"admin\")\n    end\n    case_i :x_greater_or_equal_y do\n      user_task(\"3\", group: \"admin\")\n      user_task(\"4\", group: \"admin\")\n    end\n  end\nend\n```","ref":"Mozart.BpmProcess.html#case_task/2"},{"type":"macro","title":"Mozart.BpmProcess.conditional_task/3","doc":"Used to define a task that executes only when a condition (function)\nevaluates to true (a truthy value). When  the condition evaluates to\nfalse, execution proceeds directly to the next task.\n\nExample:\n\n```\ndefprocess \"process with conditional task\" do\n  prototype_task(\"fast initial credit check\")\n  conditional_task \"additional credit check\", condition: :initial_check_inconclusive do\n    prototype_task(\"extensive credit check\")\n  end\n  prototype_task(\"report credit check results\")\nend\n```","ref":"Mozart.BpmProcess.html#conditional_task/3"},{"type":"macro","title":"Mozart.BpmProcess.def_bpm_application/2","doc":"Used to define a BPM application. An BPM application is used to identity a top level\nbusiness process implemented in a Elixir module. It's purpose is to facilitate external tool\nintegration. A single Elixir module might have zero or one such definitions. Define a BPM application\nwhen you intend for users to start process instances from a Mozart GUI.\nIt's parameters are as follows:\n* **process**: The name of the top level process defintion.\n* **data**: A keyword argument for a comma separated list of input parameters that the busines process should be initialized with.\n* **bk_prefix**: A keyword argument for a set of data values to use as the process instances business\nkey prefix.\n\nExample:\ndef_bpm_application(\"Home Loan\", data: \"Customer Name,Income,Debt\", bk_prefix: \"Customer Name\")","ref":"Mozart.BpmProcess.html#def_bpm_application/2"},{"type":"macro","title":"Mozart.BpmProcess.def_choice_type/2","doc":"Used to associate a single choice, enumerated type for a process data parameter. It's\npurpose is to facilitate interation with external tools, e.g. GUIs.\n\nExample:\n\ndef_choice_type(\"Pre Approval\", choices: \"Approved, Declined\")","ref":"Mozart.BpmProcess.html#def_choice_type/2"},{"type":"macro","title":"Mozart.BpmProcess.def_confirm_type/1","doc":"Used to associate a confirmational type for a process data parameter. It's\npurpose is to facilitate interation with external tools, e.g. GUIs. Intended\nto be associated with a check box that the user must check to complete a task.\n\nExample:\n\ndef_confirm_type(\"confirm param\")","ref":"Mozart.BpmProcess.html#def_confirm_type/1"},{"type":"macro","title":"Mozart.BpmProcess.def_multi_choice_type/2","doc":"Used to associate a multiple choice, enumerated type for a process data parameter. It's\npurpose is to facilitate interation with external tools, e.g. GUIs.\n\nExample:\n\ndef_multi_choice_type(\"multi choice param\", choices: \"foo,bar,foobar\")","ref":"Mozart.BpmProcess.html#def_multi_choice_type/2"},{"type":"macro","title":"Mozart.BpmProcess.def_number_type/2","doc":"Used to associate a numerical type for a process data parameter. It's\npurpose is to facilitate interation with external tools, e.g. GUIs.\n\nExample:\n\ndef_number_type(\"number param\", min: 0, max: 5)","ref":"Mozart.BpmProcess.html#def_number_type/2"},{"type":"macro","title":"Mozart.BpmProcess.defprocess/2","doc":"Used to implement a business process model. Arguments are a name followed by one or\nmore task functions.\n\n```\ndefprocess \"two timer task process\" do\n  timer_task(\"one second timer task\", duration: 1000)\n  timer_task(\"two second timer task\", duration: 2000)\nend\n```","ref":"Mozart.BpmProcess.html#defprocess/2"},{"type":"macro","title":"Mozart.BpmProcess.parallel_task/2","doc":"Used to define parallel execution paths. Arguments are the task name\nand a list of routes.\n\n```\ndefprocess \"two parallel routes process\" do\n  parallel_task \"a parallel task\" do\n    route do\n      user_task(\"1\", group: \"admin\")\n      user_task(\"2\", group: \"admin\")\n    end\n    route do\n      user_task(\"3\", group: \"admin\")\n      user_task(\"4\", group: \"admin\")\n    end\n  end\nend\n```","ref":"Mozart.BpmProcess.html#parallel_task/2"},{"type":"macro","title":"Mozart.BpmProcess.prototype_task/2","doc":"Used to specify a task that has no behavior. Used for prototyping. Has a single\nname argument\n\n```\ndefprocess \"two prototype task process\" do\n  prototype_task(\"prototype task 1\")\n  prototype_task(\"prototype task 2\")\nend\n```","ref":"Mozart.BpmProcess.html#prototype_task/2"},{"type":"macro","title":"Mozart.BpmProcess.receive_task/2","doc":"Used to suspend an execution path until receiving a specified PubSub event. Arguemnts\nare a task name and captured function\n\n```\ndef receive_loan_income(msg) do\n  case msg do\n    {:barrower_income, income} -> %{barrower_income: income}\n    _ -> nil\n  end\nend\n\ndefprocess \"receive barrower income process\" do\n  receive_task(\"receive barrower income\", selector: :receive_loan_income)\nend\n```","ref":"Mozart.BpmProcess.html#receive_task/2"},{"type":"macro","title":"Mozart.BpmProcess.repeat_task/3","doc":"Used to a sequence of tasks that repeat as long as a condition evaluates to\na truthy value.\n\nExample:\n```\ndefprocess \"account overdue process\" do\n  repeat_task \"notify customer\", condition: :third_notice_sent do\n    service_task(\"send overdue notice\", function: :record_notifiction_sent)\n  end\nend\n```","ref":"Mozart.BpmProcess.html#repeat_task/3"},{"type":"macro","title":"Mozart.BpmProcess.reroute_task/3","doc":"Used to reroute a process flow off the typical execution path (off the 'happy path').\n\nExample:\n```\ndefprocess \"act on one of multiple events\" do\n  prototype_task(\"create order\")\n  receive_task(\"receive payment details\", selector: :receive_payment_details)\n  reroute_task \"payment period expired\", condition: :payment_period_expired do\n    prototype_task(\"cancel order due to timeout\")\n  end\n  reroute_task \"order canceled\", condition: :order_canceled do\n    prototype_task(\"cancel order due to order cancelation\")\n  end\n  prototype_task(\"process payment\")\nend\n```","ref":"Mozart.BpmProcess.html#reroute_task/3"},{"type":"macro","title":"Mozart.BpmProcess.rule_task/2","doc":"Used to set data properties based on evaluation of rules in a rule table. Arguments\nare a task name, a set of inputs and a rule table.\n\n```\nrule_table = \"\"\"\nF     income      || status\n1     > 50000     || approved\n2     <= 49999    || declined\n\"\"\"\n\ndefprocess \"single rule task process\" do\n  rule_task(\"loan decision\", inputs: \"income\", rule_table: rule_table)\nend\n```","ref":"Mozart.BpmProcess.html#rule_task/2"},{"type":"macro","title":"Mozart.BpmProcess.send_task/2","doc":"A task that send a PubSub event to a receiving receive_task. Arguments are a task name\nand a PubSub message.\n\n```\ndefprocess \"send barrower income process\" do\n  send_task(\"send barrower income\", message: {:barrower_income, 100_000})\nend\n```","ref":"Mozart.BpmProcess.html#send_task/2"},{"type":"macro","title":"Mozart.BpmProcess.service_task/2","doc":"Used to specify a task completed by calling an Elixir function. Arguments are\na task name, a set of comma delitmited inputs a captured Elixir function.\n\n```\ndef square(data) do\n  Map.put(data, :square, data.x * data.x)\nend\n\ndefprocess \"one service task process\" do\n  service_task(\"a service task\", function: :square, inputs: \"x\")\nend\n```","ref":"Mozart.BpmProcess.html#service_task/2"},{"type":"macro","title":"Mozart.BpmProcess.subprocess_task/2","doc":"Used to specify a task completed by spawning and completing a subprocess. Arguments\nare a task name and the name of the subprocess model.\n\n```\ndefprocess \"subprocess task process\" do\n  subprocess_task(\"subprocess task\", process: \"two service tasks\")\nend\n```","ref":"Mozart.BpmProcess.html#subprocess_task/2"},{"type":"macro","title":"Mozart.BpmProcess.timer_task/2","doc":"Used to specify a timed delay in an execution path. Arguments are a task name and\na duration in milliseconds.\n\n```\ndefprocess \"two timer task process\" do\n  timer_task(\"one second timer task\", duration: 1000)\n  timer_task(\"two second timer task\", duration: 2000)\nend\n```","ref":"Mozart.BpmProcess.html#timer_task/2"},{"type":"macro","title":"Mozart.BpmProcess.user_task/2","doc":"Used to specify a task performed by a user belonging to a specified workgroup. Arguments are\na task name and a set of comma delimited workgroups.\n\n```\ndefprocess \"single user task process\" do\n  user_task(\"add one to x\", group: \"admin,customer_service\")\nend\n```","ref":"Mozart.BpmProcess.html#user_task/2"},{"type":"module","title":"Mozart.ProcessEngine","doc":"A **ProcessEngine** is dynamically spawned for the purpose of executing a top level process model defined by **defprocess** function (macro) call. Subprocess tasks do not result in the spawning of a new **ProcessEngine** instance. Instead, subprocess tasks are handled by pushing an execution frame upon the execution frame stack.","ref":"Mozart.ProcessEngine.html"},{"type":"function","title":"Mozart.ProcessEngine.execute/1","doc":"Used to complete any \"complete-able\" open tasks. Task execution frequently spawns new\nopen tasks. Execute will continue to called recursively as long as there are \"complete-able\" open tasks.","ref":"Mozart.ProcessEngine.html#execute/1"},{"type":"function","title":"Mozart.ProcessEngine.get_completed_tasks/1","doc":"Retrieves all completed tasks for a process instance.","ref":"Mozart.ProcessEngine.html#get_completed_tasks/1"},{"type":"function","title":"Mozart.ProcessEngine.get_open_tasks/1","doc":"Gets the open tasks of the given process engine. Useful for debugging.","ref":"Mozart.ProcessEngine.html#get_open_tasks/1"},{"type":"function","title":"Mozart.ProcessEngine.get_state/1","doc":"Returns the state of the process. Useful for debugging.","ref":"Mozart.ProcessEngine.html#get_state/1"},{"type":"function","title":"Mozart.ProcessEngine.start_process/3","doc":"Use this function to create a ProcessEngine instance initialized with the\nname of the process model to be executed and any initialization data. The\nengine will start executing tasks when the execute/1 function is called.\n\nArguments are a process model name, initial process data, an optional business_key, and a parent\nprocess pid if there is a parent. If a process key is not specified, one will be assigned.\n\nReturns a tuple of the form:\n```\n{:ok, ppid, uid, business_key}\n```\nwhere:\n* **ppid** is the Elixir pid for the spawned GenServer.\n* **uid** is a uniquie identifier for a process execution.\n* **business_key** is a unique identifier for a hierarchial process execution.\n\nSample invocation:\n```\n{:ok, ppid, uid, business_key} = ProcessEngine.start_process(\"a process model name\", )\n```","ref":"Mozart.ProcessEngine.html#start_process/3"},{"type":"module","title":"Mozart.ProcessService","doc":"This modeule provides a set of services for working with BPM processes.","ref":"Mozart.ProcessService.html"},{"type":"function","title":"Mozart.ProcessService.complete_user_task/2","doc":"Completes a user task.","ref":"Mozart.ProcessService.html#complete_user_task/2"},{"type":"function","title":"Mozart.ProcessService.get_active_processes/0","doc":"Get active process instances","ref":"Mozart.ProcessService.html#get_active_processes/0"},{"type":"function","title":"Mozart.ProcessService.get_bpm_application/1","doc":"Gets a BPM Application","ref":"Mozart.ProcessService.html#get_bpm_application/1"},{"type":"function","title":"Mozart.ProcessService.get_bpm_applications/0","doc":"Gets the names of all BPM applications","ref":"Mozart.ProcessService.html#get_bpm_applications/0"},{"type":"function","title":"Mozart.ProcessService.get_completed_process/1","doc":"Returns the state of the completed process corresponding to the process engine's uid","ref":"Mozart.ProcessService.html#get_completed_process/1"},{"type":"function","title":"Mozart.ProcessService.get_completed_process_data/1","doc":"Returns the data accumulated by the completed process","ref":"Mozart.ProcessService.html#get_completed_process_data/1"},{"type":"function","title":"Mozart.ProcessService.get_process_model/1","doc":"Retrieves a process model by name.","ref":"Mozart.ProcessService.html#get_process_model/1"},{"type":"function","title":"Mozart.ProcessService.get_user_tasks_for_group/1","doc":"Returns the user tasks that can be completed by users belonging to one of the input group.","ref":"Mozart.ProcessService.html#get_user_tasks_for_group/1"},{"type":"function","title":"Mozart.ProcessService.handle_continue/2","doc":"","ref":"Mozart.ProcessService.html#handle_continue/2"},{"type":"function","title":"Mozart.ProcessService.load_bpm_applications/1","doc":"Loads a BPM Application","ref":"Mozart.ProcessService.html#load_bpm_applications/1"},{"type":"function","title":"Mozart.ProcessService.load_process_model/1","doc":"Loads a single process model in the repository.","ref":"Mozart.ProcessService.html#load_process_model/1"},{"type":"function","title":"Mozart.ProcessService.load_process_models/1","doc":"Loads a list of process model into the state of the\nProcessService.","ref":"Mozart.ProcessService.html#load_process_models/1"},{"type":"extras","title":"Introduction to BPM","doc":"# Introduction to BPM\n\nThe next several sections provide an introduction to to ***Business Process Management***.","ref":"intro_bpm.html"},{"type":"extras","title":"What is a Business Process? - Introduction to BPM","doc":"Companies large and small have business processes that they regularly execute. Let's just call them \"processes\" for brevity's sake. Insurance companies have processes to process a claim. Banks have processes to approve loans. The government have processes to award contracts. \n\nBusiness processes have the following charateristics:\n\n* A business process is comprised of some number of individual tasks. \n* Tasks are performed in an order prescribed by network of task connections.\n* Processes can branch to multiple alternative sets of tasks depending on the state of the process.\n* Processes can branch to multiple parallel paths as well.\n* There are multiple kinds of tasks. Some tasks are performed by users. Some tasks are performed by calling services. There are many other kinds of tasks as well.\n* Processes store data to be used by tasks to perform their functions.\n* Processes allow tasks to add data for use by downstream tasks.","ref":"intro_bpm.html#what-is-a-business-process"},{"type":"extras","title":"What is Business Process Management (BPM)? - Introduction to BPM","doc":"Business Process Management (BPM) is the organized and directed activity of a company to control, monitor and improve the quality of their business processes. They do this with the aid of a BPM platform made available by dozens of commercial vendors.\n\nBPM platforms provide the following functions:\n\n* Provide developers with the ability of define and store process models.\n* Provide developers with the ability have certain tasks completed by making service calls.\n* Provide users and administrators with the ability to start process instances.\n* Provide users with the ability to assign user tasks to specified users.\n* Prvide users with the ability to assign user tasks to members of a specified user group.\n\nThe goals of Business Process Management are:\n\n* Better Process Reliablibility\n* Faster Process Execution\n* Reduced Training Requirements\n* Incremental Process Improvement\n* Provide Process Metrics\n* Cost Savings","ref":"intro_bpm.html#what-is-business-process-management-bpm"},{"type":"extras","title":"When is BPM Applicable - Introduction to BPM","doc":"This is difficult to answer exactly, but BPM may be applicable when:\n\n* Processes are performed on a regular basis.\n* Successful process execution is crucial to the business.\n* Current processes are error prone.\n* The speed of process completion is very important.\n* Processes are well defined.\n* Multiple business groups and/or people participate in process execution.","ref":"intro_bpm.html#when-is-bpm-applicable"},{"type":"extras","title":"Introduction to Mozart","doc":"# Introduction to Mozart","ref":"app_overview.html"},{"type":"extras","title":"Process Models - Introduction to Mozart","doc":"The primary concept of *Mozart* is the **Process Model**. A *process model* is an *executable* representation of a business process. *Process models* are implemented using a collection of functions that together comprise a **Domain Specific Language (DSL)** used for the purpose of creating BPM Applications.\n\nIf you are a developer working for a bank, you might be asked to develop a process model for handling a customer loan application. A grossly simplified process model for this might look like:\n\n```elixir\n  defprocess \"process loan application\" do\n    user_task(\"record and check loan application\",  group: \"customer service\")\n    user_task(\"determine loan acceptance\",          group: \"underwriting\")\n    user_task(\"inform customer of result\",          group: \"customer service\")\n  end\n```","ref":"app_overview.html#process-models"},{"type":"extras","title":"Process Model Tasks - Introduction to Mozart","doc":"As indicated in the example above, a *process model* is composed of a collection of **tasks**. Each task represents some work that needs to be completed. To **complete** a task means to perform the work required of the task.\n\nMozart provides around a dozen different types of *tasks*. Tasks are defined using functions in the Mozart DSL.\n\nThe task types currently supported are:\n\n| Task Type               |  Description |\n|-----|-----|\n| user_task               | Performed by a user |\n| service_task            | Performed by calling a service. |\n| subprocess_task         | Performed by calling a subprocess. |\n| timer_task              | Waits for expiration of a timer. |\n| receive_task            | Waits for an event. |\n| send_task               | Sends an event. |\n| rule_task               | Performed by evaluating a set of rules represented in a table. |\n| case_task               | Selects one of many process paths. |\n| parallel_task           | Initiates two or more parallel process paths. |\n| prototype_task          | Has no behavior. Used for prototyping and subbing. |\n| repeat_task             | Repeats a set of tasks while condition is true. |\n| reroute_task            | Reroutes a process off the typical (happy) execution path. |\n| conditional_task        | Implements a conditional task along the typical execution path. |","ref":"app_overview.html#process-model-tasks"},{"type":"extras","title":"BPM Process Module - Introduction to Mozart","doc":"A **BPM Process Module** is an Elixir module containing a set of related **process models, BPM applications, BPM related type definitions** as well as any other normal function definitions. There is an [example process application](https://github.com/CharlesIrvineKC/mozart/blob/main/test/support/home_loan_app.ex) in the Mozart GitHub repository.\n\n*Process applications* must **use Mozart.BpmProcess** to gain access to the Mozart DSL:\n\n```elixir\ndefmodule MyProcessApplication do\n  use Mozart.BpmProcess\n\n  defun send_loan_response(data) do\n    # Code to send customer and email loan resonse\n  end\n\n  defprocess \"process loan application\" do\n    user_task(\"record and check loan application\", group: \"customer service\")\n    user_task(\"determine loan acceptance\", group: \"underwriting\")\n    service_task(\"inform customer of result\", function: &MyProcessApplication.send_loan_response/1)\n  end\n\n  # multiple process model and function definitions\n\nend\n```\n\nDeclaring **use Mozart.BpmProcess** injects the **MyBpmApplication.load/0** function into your application. Calling this function will load all of your process models into the `Mozart.ProcessService` model repository. Use it now to load your process models:\n\n```elixir\nMyProcessApplication.load()\n```","ref":"app_overview.html#bpm-process-module"},{"type":"extras","title":"BPM Applications - Introduction to Mozart","doc":"Each Elixir BPM module can define zero or more **BPM Applications**. Here is an example\n\n```elixir\ndef_bpm_application(\"Home Loan\", data: \"Customer Name,Income,Debt\")\n```\nEach application specifies a user level name, the name of the top level process definition and a set of required \ninput parameters.\n\nThe purpose of a BPM application is to aid in intergration with external applications, e.g. GUI applications.","ref":"app_overview.html#bpm-applications"},{"type":"extras","title":"Parameter type specifications - Introduction to Mozart","doc":"Each Elixir BPM process module can define any number of parameter type definitions. Here are examples\nof the currently supported types:\n\n```elixir\ndef_number_type(\"number param\", min: 0, max: 5)\ndef_choice_type(\"choice param\", choices: \"foo, bar\")\ndef_multi_choice_type(\"multi choice param\", choices: \"foo,bar,foobar\")\ndef_confirm_type(\"confirm param\")\n```\n\nType definitions aid in integration with external applications, e.g. GUIs.","ref":"app_overview.html#parameter-type-specifications"},{"type":"extras","title":"Process Engine - Introduction to Mozart","doc":"We have said that a Mozart process model is **executable**. What does that mean? After loading the process model as shown above, commands are invoked to start and begin executing the process model. Here is a corresponding code snippet:\n\n```elixir\ndata = %{loan_amount_requested: 300_000}\n\n{:ok, ppid, uid, _business_key} = ProcessEngine.start_process(\"process loan application\", data)\n\nProcessEngine.execute(ppid)\n```\n\nFirst, we define some data that the process model will use. Then a *process engine* is spawned by calling **ProcessEngine.start_process/2**. The first parameter specifies the top level *process model* to be executed. As you will see later on, a hierarchy of process instances can be spawned to complete the top level process model. The second argument is data used to initiate process execution. \n\nThe first argument returned is the Elixir *process identifier* of the Elixir GenServer. The secord value returned is a unique identifier of the process execution. This will be useful beyond the lifetime of the Elixir process. Finally, the third value returned is another unique identifier corresponding to the hierarchy of processes spawned to complete the top level process model.\n\nAfter, the process engine has been spawned, process execution begins by calling **ProcessEngine.execute/1** function, passing it the Elixir PID contained in the return value bound to the variable **ppid**.","ref":"app_overview.html#process-engine"},{"type":"extras","title":"Process Service - Introduction to Mozart","doc":"We have already seen that the GenServer `Mozart.ProcessService` provides a repository for process models. It provides numerous additional functions [documented in hexdocs](https://hexdocs.pm/mozart/Mozart.ProcessService.html#summary).","ref":"app_overview.html#process-service"},{"type":"extras","title":"Processes Model Execution - Introduction to Mozart","doc":"Earlier we said that the job of the process engine is to drive a process model execution to completion. Let's discuss what that means.","ref":"app_overview.html#processes-model-execution"},{"type":"extras","title":"Completing Tasks - Introduction to Mozart","doc":"When process execution is triggered by calling **ProcessEngine.execute/1**, the first task specified in the process model will be opened for completion.\n\n```elixir\n defprocess \"process loan application\" do\n    user_task(\"record and check loan application\", group: \"customer service\")\n    user_task(\"determine loan acceptance\", group: \"underwriting\")\n    service_task(\"inform customer of result\", function: &MyProcessApplication.send_loan_response/1)\n  end\n```\n\nHence, in the process model above, the user task named **record and check loan application** will be opened for completion upon the initiation of process execution.\n\nCompletion of this initial task will typically trigger the opening of additional tasks. Every task that completes will potentially trigger the opening of additional tasks. Eventually, a process model will have no open tasks. This signals that execution of the process model is complete and the corresponding process engine will exit.\n\nSo, completing the execution of a process model equates to repeatedly opening and completing tasks until the process as a whole is complete.","ref":"app_overview.html#completing-tasks"},{"type":"extras","title":"Accumulating Data - Introduction to Mozart","doc":"Almost every task that is opened will need some data to complete. Further, when a task completes, it will introduce some new data for downstream tasks to use. Thus, to a great extent, completing a processes equates to generating a store of accumulated information.\n\nAll of the data generated by task completion is stored in the **data** property of `Mozart.ProcessEngine`.","ref":"app_overview.html#accumulating-data"},{"type":"extras","title":"Producing Side Effects - Introduction to Mozart","doc":"To be useful, almost all process model executions will produce some side effects. Take the process of processing a bank loan application for example. What are the possible side effects?\n\n* Sending a notice that the loan is approved or declined.\n* Depositing money in the customers account.\n* Setting up the loan in the banks backend system.","ref":"app_overview.html#producing-side-effects"},{"type":"extras","title":"Conclusion - Introduction to Mozart","doc":"Hopefully, the mechanics of Business Process Managment is now much clearer. In the newxt section, we will use Mozart to execute business processes.","ref":"app_overview.html#conclusion"},{"type":"extras","title":"First Process Execution","doc":"# First Process Execution\n\nNow we will put together what we've lerned so far to execute a process model.","ref":"first_process_execution.html"},{"type":"extras","title":"To Follow Along.. - First Process Execution","doc":"To follow along create a new Elxir **mix** project and then add Mozart as a dependency in your **mix.exs** project file.\n\nCreate a project:\n\n```\n$ mix new mozart_play\n```\n\nOpen mix.exs and add **mozart** dependency:\n\n```elixir\n  defp deps do\n    [\n      {:mozart, \"~> 0.9\"}\n    ]\n  end\n```\n\nGet dependencies:\n\n```\n$ mix deps.get\n```","ref":"first_process_execution.html#to-follow-along"},{"type":"extras","title":"Create BPM Application - First Process Execution","doc":"First you need to create a BPM application module. In it you will specify your process models and any needed supported functions. Add the file lib/my_bpm_application.ex to your project:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess\n\n  # Your process models and supporting functions will go here.\n\nend\n```","ref":"first_process_execution.html#create-bpm-application"},{"type":"extras","title":"Construct the Process Model - First Process Execution","doc":"First, let us construct a simple process model that will have a single service task. The service task will simply return the sum of two input properties. Update your BPM application module as follows:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess\n\n  def sum(data) do\n    %{sum: data[\"x\"] + data[\"y\"]}\n  end\n\n  defprocess \"add x and y process\" do\n    service_task(\"add x and y task\", function: :sum, inputs: \"x,y\")\n  end\n\nend\n```","ref":"first_process_execution.html#construct-the-process-model"},{"type":"extras","title":"Load the Process Model into repository - First Process Execution","doc":"Now let's load this process model into the process model repository, which is implemented by `Mozart.ProcessService`. First, start an iex session with **iex -S mix** and paste a couple of aliases into your iex session:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\n\n```\n\nNow we are ready to load our process model into the repository. Copy the following into your iex session:\n\n```elixir\nMyBpmApplication.load()\n\n```\n\nYou should see:\n\n````\niex [15:06 :: 3] > MyBpmApplication.load()\n%{\n  active_process_groups: %{},\n  active_processes: %{},\n  restart_state_cache: %{},\n  user_task_db: #PID<0.1995.0>,\n  completed_process_db: #PID<0.2000.0>,\n  process_model_db: #PID<0.2004.0>,\n  bpm_application_db: #PID<0.2008.0>,\n  process_state_db: #PID<0.2013.0>,\n  type_db: #PID<0.2017.0>\n}\n````\n\nThe function **MyBpmApplication.load/0** was created for you automatically when you inserted **use Mozart.BpmProcess** into your module definition. The output shows the internal state of the system. Disregard this for now.","ref":"first_process_execution.html#load-the-process-model-into-repository"},{"type":"extras","title":"Start a Process Engine and Execute the Process Model - First Process Execution","doc":"Now let us start a process engine, initializing it with our process model and some data. Then we call the function that will invoke process execution. Copy the following into your iex session:\n\n```elixir\n{:ok, ppid, uid, _key} = PE.start_process(\"add x and y process\",%{\"x\" => 1, \"y\" => 2})\n\n```\n\nYou should now see something like:\n\n```\n10:04:26.242 [info] Start process instance [add x and y process][a57d93d4-fc97-4a3c-89f6-989c088c96a7]\n{:ok, #PID<0.457.0>, \"a57d93d4-fc97-4a3c-89f6-989c088c96a7\", \"d41dc645-2417-4778-bb18-fccf2e6e44f6\"}\n```\n\nWe started a process instance using `Mozart.ProcessEngin.start_process/2`. We passed it the name of the process model to be executed and some initial data required for process execution. The function call returned a tuple with three values: \n\n* The **pid** of the resulting process engine.\n* The **uid** of the process instance. Every process execution will be given a unique identifier. You will see how this is used going forward.\n* Finally, a **business_key** is returned. Process models are hierarchial, meaning they can be composed of multiple, nested subprocesses. The top level business process and all of its subprocesses will share the same *business_key*. Here, were aren't going to be using the *process key*, but we will in subsequent examples.\n\nAfter creating the process engine instance, we then call `Mozart.ProcessEngine.execute/1` which causes the process engine to start completing any tasks that are ready to be completed. Let's do that now:\n\n```\nPE.execute(ppid)\n\n```\n\nAfter calling the *ProcessEngine.execute* function, you should see something like:\n\n```elixir\niex [08:50 :: 23] > PE.execute(ppid)\n:ok\n10:14:43.450 [info] New service task instance [add x and y task][2dd65cc0-ca74-4b4d-9219-db36e486dd68]\n10:14:43.454 [info] Complete service task [add x and y task[2dd65cc0-ca74-4b4d-9219-db36e486dd68]\n10:14:43.456 [info] Exit process: process complete [add x and y process][3261eb1a-7b2a-438a-aa2a-3c9ec646e8f7]\n```\n\nThe logs show that:\n\n1. A new service task was opened.\n1. The service task completed.\n1. And finally the process engine completed executing the process model.","ref":"first_process_execution.html#start-a-process-engine-and-execute-the-process-model"},{"type":"extras","title":"Verify the Results - First Process Execution","doc":"Let's verify the results of our process model execution by calling **ProcessService.get_completed_process_data/1**. Copy the following into your iex session:\n\n```elixir\nPS.get_completed_process_data(uid)\n\n```\n\nand you should see:\n\n```elixir\niex [08:50 :: 26] > PS.get_completed_process_data(uid)\n%{:sum => 3, \"x\" => 1, \"y\" => 2}\n```\n\nWe see that a **sum** property was added to process data whose value is the sume of the properties **x** and **y**..","ref":"first_process_execution.html#verify-the-results"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,51,46,48]}}